"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import amountdiscount as shared_amountdiscount
from ..shared import bpsprice as shared_bpsprice
from ..shared import bulkbpsprice as shared_bulkbpsprice
from ..shared import bulkprice as shared_bulkprice
from ..shared import fivetranexampleprice as shared_fivetranexampleprice
from ..shared import matrixprice as shared_matrixprice
from ..shared import maximum as shared_maximum
from ..shared import minimum as shared_minimum
from ..shared import packageprice as shared_packageprice
from ..shared import packagewithallocationprice as shared_packagewithallocationprice
from ..shared import percentagediscount as shared_percentagediscount
from ..shared import planminified as shared_planminified
from ..shared import planphase as shared_planphase
from ..shared import product as shared_product
from ..shared import testratingfunctionprice as shared_testratingfunctionprice
from ..shared import thresholdtotalamountprice as shared_thresholdtotalamountprice
from ..shared import tieredbpsprice as shared_tieredbpsprice
from ..shared import tieredpackageprice as shared_tieredpackageprice
from ..shared import tieredprice as shared_tieredprice
from ..shared import tieredwithminimumprice as shared_tieredwithminimumprice
from ..shared import trialconfig as shared_trialconfig
from ..shared import trialdiscount as shared_trialdiscount
from ..shared import unitprice as shared_unitprice
from ..shared import usagediscount as shared_usagediscount
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class PlanDiscount:
    pass



@dataclasses.dataclass
class PlanMetadata:
    pass

class PlanStatus(str, Enum):
    ACTIVE = 'active'
    ARCHIVED = 'archived'
    DRAFT = 'draft'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Plan:
    r"""The [Plan](../guides/core-concepts.mdx#plan-and-price) resource represents a plan that can be subscribed to by a
    customer. Plans define the billing behavior of the subscription. You can see more about how to configure prices
    in the [Price resource](/reference/price).
    """
    base_plan: Optional[shared_planminified.PlanMinified] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('base_plan') }})
    base_plan_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('base_plan_id') }})
    r"""The parent plan id if the given plan was created by overriding one or more of the parent's prices"""
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency') }})
    r"""An ISO 4217 currency string or custom pricing unit (`credits`) for this plan's prices."""
    default_invoice_memo: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('default_invoice_memo') }})
    r"""The default memo text on the invoices corresponding to subscriptions on this plan. Note that each subscription may configure its own memo."""
    description: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    discount: Optional[Union[shared_percentagediscount.PercentageDiscount, shared_trialdiscount.TrialDiscount, shared_usagediscount.UsageDiscount, shared_amountdiscount.AmountDiscount]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount') }})
    external_plan_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_plan_id') }})
    r"""An optional user-defined ID for this plan resource, used throughout the system as an alias for this Plan. Use this field to identify a plan by an existing identifier in your system."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    invoicing_currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoicing_currency') }})
    r"""An ISO 4217 currency string for which this plan is billed in. Matches `currency` unless `currency` is a custom pricing unit."""
    maximum: Optional[shared_maximum.Maximum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum') }})
    maximum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    metadata: PlanMetadata = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    minimum: Optional[shared_minimum.Minimum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum') }})
    minimum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    net_terms: Optional[int] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    r"""Determines the difference between the invoice issue date and the due date. A value of \\"0\\" here signifies that invoices are due on issue, whereas a value of \\"30\\" means that the customer has a month to pay the invoice before its overdue. Note that individual subscriptions or invoices may set a different net terms configuration."""
    plan_phases: Optional[list[shared_planphase.PlanPhase]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('plan_phases') }})
    prices: list[Union[shared_unitprice.UnitPrice, shared_packageprice.PackagePrice, shared_matrixprice.MatrixPrice, shared_tieredprice.TieredPrice, shared_tieredbpsprice.TieredBpsPrice, shared_bpsprice.BpsPrice, shared_bulkbpsprice.BulkBpsPrice, shared_bulkprice.BulkPrice, shared_testratingfunctionprice.TestRatingFunctionPrice, shared_fivetranexampleprice.FivetranExamplePrice, shared_thresholdtotalamountprice.ThresholdTotalAmountPrice, shared_tieredpackageprice.TieredPackagePrice, shared_tieredwithminimumprice.TieredWithMinimumPrice, shared_packagewithallocationprice.PackageWithAllocationPrice]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('prices') }})
    r"""Prices for this plan. If the plan has phases, this includes prices across all phases of the plan."""
    product: shared_product.Product = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('product') }})
    status: PlanStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    trial_config: shared_trialconfig.TrialConfig = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_config') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class IncrementCreditLedgerEntryInvoiceSettings:
    r"""Passing `invoice_settings` automatically generates an invoice for the newly added credits. If `invoice_settings` is passed, you must specify per_unit_cost_basis, as the calculation of the invoice total is done on that basis."""
    auto_collection: bool = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('auto_collection') }})
    r"""Whether the credits purchase invoice should auto collect with the customer's saved payment method."""
    net_terms: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    r"""The net terms determines the difference between the invoice date and the issue date for the invoice. If you intend the invoice to be due on issue, set this to 0."""
    memo: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('memo') }})
    r"""An optional memo to display on the invoice."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import amountdiscountcreationparams as shared_amountdiscountcreationparams
from ..shared import newbpsprice as shared_newbpsprice
from ..shared import newbulkbpsprice as shared_newbulkbpsprice
from ..shared import newbulkprice as shared_newbulkprice
from ..shared import newmatrixprice as shared_newmatrixprice
from ..shared import newpackageprice as shared_newpackageprice
from ..shared import newpackagewithallocationprice as shared_newpackagewithallocationprice
from ..shared import newthresholdtotalamountprice as shared_newthresholdtotalamountprice
from ..shared import newtieredbpsprice as shared_newtieredbpsprice
from ..shared import newtieredpackageprice as shared_newtieredpackageprice
from ..shared import newtieredprice as shared_newtieredprice
from ..shared import newtieredwithminimumprice as shared_newtieredwithminimumprice
from ..shared import newunitprice as shared_newunitprice
from ..shared import percentagediscountcreationparams as shared_percentagediscountcreationparams
from ..shared import priceintervalfixedfeequantitytransitionparams as shared_priceintervalfixedfeequantitytransitionparams
from ..shared import usagediscountcreationparams as shared_usagediscountcreationparams
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class AddPriceIntervalParamsDiscounts:
    pass

class AddPriceIntervalParamsEndDateEndDate(str, Enum):
    r"""The end date of the price interval. This is the date that the price will stop billing on the subscription."""
    START_OF_TERM = 'start_of_term'
    END_OF_TERM = 'end_of_term'



@dataclasses.dataclass
class AddPriceIntervalParamsEndDate:
    pass

class AddPriceIntervalParamsStartDateStartDate(str, Enum):
    r"""The start date of the price interval. This is the date that the price will start billing on the subscription."""
    START_OF_TERM = 'start_of_term'
    END_OF_TERM = 'end_of_term'



@dataclasses.dataclass
class AddPriceIntervalParamsStartDate:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AddPriceIntervalParams:
    start_date: Union[datetime, AddPriceIntervalParamsStartDateStartDate] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date') }})
    r"""The start date of the price interval. This is the date that the price will start billing on the subscription."""
    discounts: Optional[list[Union[shared_amountdiscountcreationparams.AmountDiscountCreationParams, shared_percentagediscountcreationparams.PercentageDiscountCreationParams, shared_usagediscountcreationparams.UsageDiscountCreationParams]]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discounts') }})
    r"""A list of discounts to initialize on the price interval."""
    end_date: Optional[Union[datetime, AddPriceIntervalParamsEndDateEndDate]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date') }})
    r"""The end date of the price interval. This is the date that the price will stop billing on the subscription."""
    external_price_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_price_id') }})
    r"""The external price id of the price to add to the subscription."""
    fixed_fee_quantity_transitions: Optional[list[shared_priceintervalfixedfeequantitytransitionparams.PriceIntervalFixedFeeQuantityTransitionParams]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('fixed_fee_quantity_transitions') }})
    r"""A list of fixed fee quantity transitions to initialize on the price interval."""
    maximum_amount: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    r"""The maximum amount that will be billed for this price interval for a given billing period."""
    minimum_amount: Optional[float] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    r"""The minimum amount that will be billed for this price interval for a given billing period."""
    price: Optional[Union[shared_newunitprice.NewUnitPrice, shared_newpackageprice.NewPackagePrice, shared_newmatrixprice.NewMatrixPrice, shared_newtieredprice.NewTieredPrice, shared_newtieredbpsprice.NewTieredBpsPrice, shared_newbpsprice.NewBpsPrice, shared_newbulkbpsprice.NewBulkBpsPrice, shared_newbulkprice.NewBulkPrice, shared_newthresholdtotalamountprice.NewThresholdTotalAmountPrice, shared_newtieredpackageprice.NewTieredPackagePrice, shared_newtieredwithminimumprice.NewTieredWithMinimumPrice, shared_newpackagewithallocationprice.NewPackageWithAllocationPrice]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price'), 'decoder': utils.decoder_with_discriminator('model_type') }})
    r"""The definition of a new price to create and add to the subscription."""
    price_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price_id') }})
    r"""The id of the price to add to the subscription."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CouponRedemption:
    coupon_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('coupon_id') }})
    end_date: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    start_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional

class AccountingProviderProviderType(str, Enum):
    QUICKBOOKS = 'quickbooks'
    NETSUITE = 'netsuite'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AccountingProvider:
    external_provider_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_provider_id') }})
    provider_type: AccountingProviderProviderType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('provider_type') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import billablemetricsimple as shared_billablemetricsimple
from ..shared import metricgroup as shared_metricgroup
from ..shared import usage as shared_usage
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils

class GroupedSubscriptionUsageRecordViewMode(str, Enum):
    PERIODIC = 'periodic'
    CUMULATIVE = 'cumulative'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class GroupedSubscriptionUsageRecord:
    billable_metric: shared_billablemetricsimple.BillableMetricSimple = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billable_metric') }})
    metric_group: shared_metricgroup.MetricGroup = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metric_group') }})
    usage: list[shared_usage.Usage] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('usage') }})
    view_mode: GroupedSubscriptionUsageRecordViewMode = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('view_mode') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Union



@dataclasses.dataclass
class AmountDiscountCreationParamsAmountDiscount:
    pass

class AmountDiscountCreationParamsDiscountType(str, Enum):
    AMOUNT = 'amount'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AmountDiscountCreationParams:
    amount_discount: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount_discount') }})
    r"""Only available if discount_type is `amount`."""
    discount_type: AmountDiscountCreationParamsDiscountType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount_type') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import backfillstatus as shared_backfillstatus
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Backfill:
    r"""A backfill represents an update to historical usage data, adding or replacing events in a timeframe."""
    close_time: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('close_time'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""If in the future, the time at which the backfill will automatically close. If in the past, the time at which the backfill was closed."""
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    customer_id: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customer_id') }})
    r"""The customer ID this backfill is scoped to. If null, this backfill is not scoped to a single customer."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    reverted_at: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('reverted_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The time at which this backfill was reverted."""
    status: shared_backfillstatus.BackfillStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    r"""The status of the backfill."""
    timeframe_end: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('timeframe_end'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    timeframe_start: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('timeframe_start'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import itemexternalconnection as shared_itemexternalconnection
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Item:
    r"""The Item resource represents a sellable product or good. Items are associated with all line items, billable metrics,
    and prices and are used for defining external sync behavior for invoices and tax calculation purposes.
    """
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    external_connections: list[shared_itemexternalconnection.ItemExternalConnection] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_connections') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional



@dataclasses.dataclass
class IdempotencyConflictHeaders:
    pass

class IdempotencyConflictStatus(int, Enum):
    FOUR_HUNDRED_AND_NINE = 409

class IdempotencyConflictType(str, Enum):
    HTTPS_DOCS_WITHORB_COM_REFERENCE_ERROR_RESPONSES_NUMBER_409_RESOURCE_CONFLICT = 'https://docs.withorb.com/reference/error-responses#409-resource-conflict'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class IdempotencyConflict:
    status: IdempotencyConflictStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    type: IdempotencyConflictType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    detail: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('detail') }})
    headers: Optional[IdempotencyConflictHeaders] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('headers') }})
    title: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('title') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import amountdiscountinterval as shared_amountdiscountinterval
from ..shared import couponredemption as shared_couponredemption
from ..shared import customer as shared_customer
from ..shared import fixedfeequantityscheduleentry as shared_fixedfeequantityscheduleentry
from ..shared import maximuminterval as shared_maximuminterval
from ..shared import minimuminterval as shared_minimuminterval
from ..shared import percentagediscountinterval as shared_percentagediscountinterval
from ..shared import plan as shared_plan
from ..shared import priceinterval as shared_priceinterval
from ..shared import subscriptiontrialinfo as shared_subscriptiontrialinfo
from ..shared import usagediscountinterval as shared_usagediscountinterval
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class SubscriptionDiscountIntervals:
    pass



@dataclasses.dataclass
class SubscriptionMetadata:
    r"""User specified key-value pairs. If no metadata was specified at subscription creation time, this defaults to an empty dictionary."""
    


class SubscriptionStatus(str, Enum):
    ACTIVE = 'active'
    ENDED = 'ended'
    UPCOMING = 'upcoming'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Subscription:
    r"""A [subscription](../guides/core-concepts.mdx#subscription) represents the purchase of a plan by a customer.

    By default, subscriptions begin on the day that they're created and renew automatically for each billing cycle at
    the cadence that's configured in the plan definition.

    Subscriptions also default to **beginning of month alignment**, which means the first invoice issued for the
    subscription will have pro-rated charges between the `start_date` and the first of the following month. Subsequent
    billing periods will always start and end on a month boundary (e.g. subsequent month starts for monthly billing).

    Depending on the plan configuration, any _flat_ recurring fees will be billed either at the beginning (in-advance)
    or end (in-arrears) of each billing cycle. Plans default to **in-advance billing**. Usage-based fees are billed in
    arrears as usage is accumulated. In the normal course of events, you can expect an invoice to contain usage-based
    charges for the previous period, and a recurring fee for the following period.
    """
    active_plan_phase_order: Optional[int] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('active_plan_phase_order') }})
    r"""The current plan phase that is active, only if the subscription's plan has phases."""
    auto_collection: Optional[bool] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('auto_collection') }})
    r"""Determines whether issued invoices for this subscription will automatically be charged with the saved payment method on the due date. This property defaults to the plan's behavior."""
    billing_cycle_day: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billing_cycle_day') }})
    r"""The day of the month on which the billing cycle is anchored. If the maximum number of days in a month is greater than this value, the last day of the month is the billing cycle day (e.g. billing_cycle_day=31 for April means the billing period begins on the 30th."""
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    current_billing_period_end_date: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('current_billing_period_end_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The end of the current billing period. This is an exclusive timestamp, such that the instant returned is not part of the billing period. Set to null for subscriptions that are not currently active."""
    current_billing_period_start_date: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('current_billing_period_start_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The start date of the current billing period. This is an inclusive timestamp; the instant returned is exactly the beginning of the billing period. Set to null if the subscription is not currently active."""
    customer: shared_customer.Customer = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customer') }})
    r"""A customer is a buyer of your products, and the other party to the billing relationship.

    In Orb, customers are assigned system generated identifiers automatically, but it's often desirable to have these
    match existing identifiers in your system. To avoid having to denormalize Orb ID information, you can pass in an
    `external_customer_id` with your own identifier. See
    [Customer ID Aliases](../guides/events-and-metrics/customer-aliases) for further information about how these
    aliases work in Orb.

    In addition to having an identifier in your system, a customer may exist in a payment provider solution like
    Stripe. Use the `payment_provider_id` and the `payment_provider` enum field to express this mapping.

    A customer also has a timezone (from the standard [IANA timezone database](https://www.iana.org/time-zones)), which
    defaults to your account's timezone. See [Timezone localization](../guides/product-catalog/timezones.md) for
    information on what this timezone parameter influences within Orb.
    """
    default_invoice_memo: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('default_invoice_memo') }})
    r"""Determines the default memo on this subscriptions' invoices. Note that if this is not provided, it is determined by the plan configuration."""
    discount_intervals: list[Union[shared_amountdiscountinterval.AmountDiscountInterval, shared_percentagediscountinterval.PercentageDiscountInterval, shared_usagediscountinterval.UsageDiscountInterval]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount_intervals') }})
    r"""The discount intervals for this subscription."""
    end_date: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The date Orb stops billing for this subscription."""
    fixed_fee_quantity_schedule: list[shared_fixedfeequantityscheduleentry.FixedFeeQuantityScheduleEntry] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('fixed_fee_quantity_schedule') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    invoicing_threshold: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoicing_threshold') }})
    maximum_intervals: list[shared_maximuminterval.MaximumInterval] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_intervals') }})
    r"""The maximum intervals for this subscription."""
    metadata: SubscriptionMetadata = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    r"""User specified key-value pairs. If no metadata was specified at subscription creation time, this defaults to an empty dictionary."""
    minimum_intervals: list[shared_minimuminterval.MinimumInterval] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_intervals') }})
    r"""The minimum intervals for this subscription."""
    net_terms: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    r"""Determines the difference between the invoice issue date for subscription invoices as the date that they are due. A value of `0` here represents that the invoice is due on issue, whereas a value of `30` represents that the customer has a month to pay the invoice."""
    plan: shared_plan.Plan = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('plan') }})
    r"""The [Plan](../guides/core-concepts.mdx#plan-and-price) resource represents a plan that can be subscribed to by a
    customer. Plans define the billing behavior of the subscription. You can see more about how to configure prices
    in the [Price resource](/reference/price).
    """
    price_intervals: list[shared_priceinterval.PriceInterval] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price_intervals') }})
    r"""The price intervals for this subscription."""
    redeemed_coupon: Optional[shared_couponredemption.CouponRedemption] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('redeemed_coupon') }})
    start_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The date Orb starts billing for this subscription."""
    status: SubscriptionStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    trial_info: shared_subscriptiontrialinfo.SubscriptionTrialInfo = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_info') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional, Union

class TrialDiscountDiscountType(str, Enum):
    TRIAL = 'trial'



@dataclasses.dataclass
class TrialDiscountTrialPercentageDiscount:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class TrialDiscount:
    applies_to_price_ids: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('applies_to_price_ids') }})
    r"""List of price_ids that this discount applies to. For plan/plan phase discounts, this can be a subset of prices."""
    discount_type: TrialDiscountDiscountType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount_type') }})
    trial_amount_discount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_amount_discount') }})
    r"""Only available if discount_type is `trial`"""
    trial_percentage_discount: Optional[Union[float, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_percentage_discount') }})
    r"""Only available if discount_type is `trial`"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class ValidationError:
    idempotency_key: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('idempotency_key') }})
    r"""The passed idempotency_key corresponding to the validation_errors"""
    validation_errors: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('validation_errors') }})
    r"""An array of strings corresponding to validation failures for this idempotency_key."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from datetime import date
from enum import Enum
from orb import utils
from typing import Any, Union

class InvoiceLineItemParamsModelType(str, Enum):
    UNIT = 'unit'



@dataclasses.dataclass
class InvoiceLineItemParamsQuantity:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class InvoiceLineItemParams:
    end_date: date = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A date string to specify the line item's end date in the customer's timezone."""
    item_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('item_id') }})
    model_type: InvoiceLineItemParamsModelType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('model_type') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""The name of the line item."""
    quantity: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity') }})
    r"""The number of units on the line item"""
    start_date: date = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.dateisoformat(False), 'decoder': utils.datefromisoformat }})
    r"""A date string to specify the line item's start date in the customer's timezone."""
    unit_config: dict[str, Any] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('unit_config') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ..shared import amendedevent as shared_amendedevent
from ..shared import amendeventresult as shared_amendeventresult
from typing import Optional



@dataclasses.dataclass
class AmendEventRequest:
    event_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'event_id', 'style': 'simple', 'explode': False }})
    amended_event: Optional[shared_amendedevent.AmendedEvent] = dataclasses.field(default=None, metadata={'request': { 'media_type': 'application/json' }})
    




@dataclasses.dataclass
class AmendEventResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    amend_event_result: Optional[shared_amendeventresult.AmendEventResult] = dataclasses.field(default=None)
    r"""OK"""
    raw_response: Optional[requests_http.Response] = dataclasses.field(default=None)
    r"""Raw HTTP response; suitable for custom response parsing"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional

class TrialConfigTrialPeriodUnit(str, Enum):
    DAYS = 'days'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class TrialConfig:
    trial_period: Optional[int] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_period') }})
    trial_period_unit: TrialConfigTrialPeriodUnit = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('trial_period_unit') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .sdkconfiguration import SDKConfiguration
from orb import utils
from orb.models import errors, operations, shared
from typing import Optional

class Availability:
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    def ping(self) -> operations.PingResponse:
        r"""Check availability
        This endpoint allows you to test your connection to the Orb API and check the validity of your API key, passed in
        the Authorization header. This is particularly useful for checking that your environment is set up properly, and is
        a great choice for connectors and integrations.

        This API does not have any side-effects or return any Orb resources.
        """
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/ping'
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.PingResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.PingResponse])
                res.ping_response = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    


"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import amountdiscount as shared_amountdiscount
from ..shared import maximum as shared_maximum
from ..shared import minimum as shared_minimum
from ..shared import percentagediscount as shared_percentagediscount
from ..shared import trialdiscount as shared_trialdiscount
from ..shared import usagediscount as shared_usagediscount
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class PlanPhaseDiscount:
    pass

class PlanPhaseDurationUnit(str, Enum):
    DAILY = 'daily'
    MONTHLY = 'monthly'
    QUARTERLY = 'quarterly'
    ANNUAL = 'annual'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PlanPhase:
    description: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    discount: Optional[Union[shared_percentagediscount.PercentageDiscount, shared_trialdiscount.TrialDiscount, shared_usagediscount.UsageDiscount, shared_amountdiscount.AmountDiscount]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount') }})
    duration: Optional[int] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duration') }})
    r"""How many terms of length `duration_unit` this phase is active for. If null, this phase is evergreen and active indefinitely"""
    duration_unit: Optional[PlanPhaseDurationUnit] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duration_unit') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    maximum: Optional[shared_maximum.Maximum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum') }})
    maximum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    minimum: Optional[shared_minimum.Minimum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum') }})
    minimum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    order: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('order') }})
    r"""Determines the ordering of the phase in a plan's lifecycle. 1 = first phase."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import amountdiscount as shared_amountdiscount
from ..shared import bpsprice as shared_bpsprice
from ..shared import bulkbpsprice as shared_bulkbpsprice
from ..shared import bulkprice as shared_bulkprice
from ..shared import fivetranexampleprice as shared_fivetranexampleprice
from ..shared import matrixprice as shared_matrixprice
from ..shared import matrixsublineitem as shared_matrixsublineitem
from ..shared import maximum as shared_maximum
from ..shared import minimum as shared_minimum
from ..shared import othersublineitem as shared_othersublineitem
from ..shared import packageprice as shared_packageprice
from ..shared import packagewithallocationprice as shared_packagewithallocationprice
from ..shared import percentagediscount as shared_percentagediscount
from ..shared import taxamount as shared_taxamount
from ..shared import testratingfunctionprice as shared_testratingfunctionprice
from ..shared import thresholdtotalamountprice as shared_thresholdtotalamountprice
from ..shared import tieredbpsprice as shared_tieredbpsprice
from ..shared import tieredpackageprice as shared_tieredpackageprice
from ..shared import tieredprice as shared_tieredprice
from ..shared import tieredwithminimumprice as shared_tieredwithminimumprice
from ..shared import tiersublineitem as shared_tiersublineitem
from ..shared import trialdiscount as shared_trialdiscount
from ..shared import unitprice as shared_unitprice
from ..shared import usagediscount as shared_usagediscount
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class InvoiceLineItemDiscount:
    pass



@dataclasses.dataclass
class InvoiceLineItemQuantity:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class InvoiceLineItem:
    amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount') }})
    r"""The final amount after any discounts or minimums."""
    discount: Optional[Union[shared_percentagediscount.PercentageDiscount, shared_trialdiscount.TrialDiscount, shared_usagediscount.UsageDiscount, shared_amountdiscount.AmountDiscount]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount') }})
    end_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The end date of the range of time applied for this line item's price."""
    grouping: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('grouping') }})
    r"""[DEPRECATED] For configured prices that are split by a grouping key, this will be populated with the key and a value. The `amount` and `subtotal` will be the values for this particular grouping."""
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    r"""A unique ID for this line item."""
    maximum: Optional[shared_maximum.Maximum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum') }})
    maximum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    minimum: Optional[shared_minimum.Minimum] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum') }})
    minimum_amount: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""The name of the price associated with this line item."""
    price: Optional[Union[shared_unitprice.UnitPrice, shared_packageprice.PackagePrice, shared_matrixprice.MatrixPrice, shared_tieredprice.TieredPrice, shared_tieredbpsprice.TieredBpsPrice, shared_bpsprice.BpsPrice, shared_bulkbpsprice.BulkBpsPrice, shared_bulkprice.BulkPrice, shared_testratingfunctionprice.TestRatingFunctionPrice, shared_fivetranexampleprice.FivetranExamplePrice, shared_thresholdtotalamountprice.ThresholdTotalAmountPrice, shared_tieredpackageprice.TieredPackagePrice, shared_tieredwithminimumprice.TieredWithMinimumPrice, shared_packagewithallocationprice.PackageWithAllocationPrice]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price'), 'decoder': utils.decoder_with_discriminator('model_type') }})
    quantity: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity') }})
    start_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The start date of the range of time applied for this line item's price."""
    sub_line_items: list[Union[shared_matrixsublineitem.MatrixSubLineItem, shared_tiersublineitem.TierSubLineItem, shared_othersublineitem.OtherSubLineItem]] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('sub_line_items') }})
    r"""For complex pricing structures, the line item can be broken down further in `sub_line_items`."""
    subtotal: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('subtotal') }})
    r"""The line amount before any line item-specific discounts or minimums."""
    tax_amounts: list[shared_taxamount.TaxAmount] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('tax_amounts') }})
    r"""An array of tax rates and their incurred tax amounts. Empty if no tax integration is configured."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class MaximumInterval:
    applies_to_price_ids: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('applies_to_price_ids') }})
    r"""The price ids that this maximum interval applies to."""
    applies_to_price_interval_ids: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('applies_to_price_interval_ids') }})
    r"""The price interval ids that this maximum interval applies to."""
    end_date: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The end date of the maximum interval."""
    maximum_amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    r"""The maximum amount to charge in a given billing period for the price intervals this transform applies to."""
    start_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""The start date of the maximum interval."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AmendedUsage:
    duplicate: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('duplicate') }})
    r"""An array of strings, corresponding to idempotency_key's marked as duplicates (previously ingested)"""
    ingested: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ingested') }})
    r"""An array of strings, corresponding to idempotency_key's which were successfully ingested."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ..shared import bpsprice as shared_bpsprice
from ..shared import bulkbpsprice as shared_bulkbpsprice
from ..shared import bulkprice as shared_bulkprice
from ..shared import fivetranexampleprice as shared_fivetranexampleprice
from ..shared import matrixprice as shared_matrixprice
from ..shared import packageprice as shared_packageprice
from ..shared import packagewithallocationprice as shared_packagewithallocationprice
from ..shared import testratingfunctionprice as shared_testratingfunctionprice
from ..shared import thresholdtotalamountprice as shared_thresholdtotalamountprice
from ..shared import tieredbpsprice as shared_tieredbpsprice
from ..shared import tieredpackageprice as shared_tieredpackageprice
from ..shared import tieredprice as shared_tieredprice
from ..shared import tieredwithminimumprice as shared_tieredwithminimumprice
from ..shared import unitprice as shared_unitprice
from typing import Optional, Union



@dataclasses.dataclass
class FetchPriceRequest:
    price_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'price_id', 'style': 'simple', 'explode': False }})
    




@dataclasses.dataclass
class FetchPriceResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    price: Optional[Union[shared_unitprice.UnitPrice, shared_packageprice.PackagePrice, shared_matrixprice.MatrixPrice, shared_tieredprice.TieredPrice, shared_tieredbpsprice.TieredBpsPrice, shared_bpsprice.BpsPrice, shared_bulkbpsprice.BulkBpsPrice, shared_bulkprice.BulkPrice, shared_testratingfunctionprice.TestRatingFunctionPrice, shared_fivetranexampleprice.FivetranExamplePrice, shared_thresholdtotalamountprice.ThresholdTotalAmountPrice, shared_tieredpackageprice.TieredPackagePrice, shared_tieredwithminimumprice.TieredWithMinimumPrice, shared_packagewithallocationprice.PackageWithAllocationPrice]] = dataclasses.field(default=None)
    r"""OK"""
    raw_response: Optional[requests_http.Response] = dataclasses.field(default=None)
    r"""Raw HTTP response; suitable for custom response parsing"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional

class CancelSubscriptionParamsCancelOption(str, Enum):
    r"""Determines the timing of subscription cancellation"""
    END_OF_SUBSCRIPTION_TERM = 'end_of_subscription_term'
    IMMEDIATE = 'immediate'
    REQUESTED_DATE = 'requested_date'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CancelSubscriptionParams:
    cancel_option: CancelSubscriptionParamsCancelOption = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('cancel_option') }})
    r"""Determines the timing of subscription cancellation"""
    cancellation_date: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('cancellation_date'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse }})
    r"""The date that the cancellation should take effect. This parameter can only be passed if the `cancel_option` is `requested_date`."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import affectedblock as shared_affectedblock
from ..shared import customerminified as shared_customerminified
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class VoidInitiatedLedgerEntryAmount:
    pass



@dataclasses.dataclass
class VoidInitiatedLedgerEntryEndingBalance:
    pass

class VoidInitiatedLedgerEntryEntryStatus(str, Enum):
    COMMITTED = 'committed'
    PENDING = 'pending'

class VoidInitiatedLedgerEntryEntryType(str, Enum):
    VOID_INITIATED = 'void_initiated'



@dataclasses.dataclass
class VoidInitiatedLedgerEntryStartingBalance:
    pass



@dataclasses.dataclass
class VoidInitiatedLedgerEntryVoidAmount:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class VoidInitiatedLedgerEntry:
    r"""The [Credit Ledger Entry resource](/guides/product-catalog/prepurchase) models prepaid credits within Orb."""
    amount: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount') }})
    created_at: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('created_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    credit_block: shared_affectedblock.AffectedBlock = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('credit_block') }})
    customer: shared_customerminified.CustomerMinified = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customer') }})
    description: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    ending_balance: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ending_balance') }})
    entry_status: VoidInitiatedLedgerEntryEntryStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('entry_status') }})
    entry_type: VoidInitiatedLedgerEntryEntryType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('entry_type') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    ledger_sequence_number: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('ledger_sequence_number') }})
    metadata: dict[str, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    new_block_expiry_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('new_block_expiry_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    starting_balance: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('starting_balance') }})
    void_amount: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('void_amount') }})
    void_reason: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('void_reason') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ..shared import creditledgerentries as shared_creditledgerentries
from datetime import datetime
from enum import Enum
from typing import Optional

class FetchCustomerCreditsLedgerExternalIDEntryStatusEntryStatus(str, Enum):
    COMMITTED = 'committed'
    PENDING = 'pending'

class FetchCustomerCreditsLedgerExternalIDEntryTypeEntryType(str, Enum):
    INCREMENT = 'increment'
    DECREMENT = 'decrement'
    EXPIRATION_CHANGE = 'expiration_change'
    CREDIT_BLOCK_EXPIRY = 'credit_block_expiry'
    VOID = 'void'
    VOID_INITIATED = 'void_initiated'
    AMENDMENT = 'amendment'



@dataclasses.dataclass
class FetchCustomerCreditsLedgerExternalIDRequest:
    external_customer_id: Optional[str] = dataclasses.field(metadata={'path_param': { 'field_name': 'external_customer_id', 'style': 'simple', 'explode': False }})
    created_at_gt: Optional[datetime] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'created_at[gt]', 'style': 'form', 'explode': True }})
    created_at_gte: Optional[datetime] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'created_at[gte]', 'style': 'form', 'explode': True }})
    created_at_lt: Optional[datetime] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'created_at[lt]', 'style': 'form', 'explode': True }})
    created_at_lte: Optional[datetime] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'created_at[lte]', 'style': 'form', 'explode': True }})
    currency: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'currency', 'style': 'form', 'explode': True }})
    cursor: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'cursor', 'style': 'form', 'explode': True }})
    entry_status: Optional[FetchCustomerCreditsLedgerExternalIDEntryStatusEntryStatus] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'entry_status', 'style': 'form', 'explode': True }})
    entry_type: Optional[FetchCustomerCreditsLedgerExternalIDEntryTypeEntryType] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'entry_type', 'style': 'form', 'explode': True }})
    limit: Optional[int] = dataclasses.field(default=20, metadata={'query_param': { 'field_name': 'limit', 'style': 'form', 'explode': True }})
    minimum_amount: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'minimum_amount', 'style': 'form', 'explode': True }})
    




@dataclasses.dataclass
class FetchCustomerCreditsLedgerExternalIDResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    credit_ledger_entries: Optional[shared_creditledgerentries.CreditLedgerEntries] = dataclasses.field(default=None)
    r"""OK"""
    raw_response: Optional[requests_http.Response] = dataclasses.field(default=None)
    r"""Raw HTTP response; suitable for custom response parsing"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import item as shared_item
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional

class BillableMetricStatus(str, Enum):
    ACTIVE = 'active'
    DRAFT = 'draft'
    ARCHIVED = 'archived'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class BillableMetric:
    r"""The Metric resource represents a calculation of a quantity based on events.
    Metrics are defined by the query that transforms raw usage events into meaningful values for your customers.
    """
    description: Optional[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('description') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    item: shared_item.Item = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('item') }})
    r"""The Item resource represents a sellable product or good. Items are associated with all line items, billable metrics,
    and prices and are used for defining external sync behavior for invoices and tax calculation purposes.
    """
    metadata: dict[str, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    status: BillableMetricStatus = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('status') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional



@dataclasses.dataclass
class NewPlanMetadata:
    pass



@dataclasses.dataclass
class NewPlanPrices:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class NewPlan:
    currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency') }})
    r"""An ISO 4217 currency string or custom pricing unit (`credits`) for this plan's prices."""
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    prices: list[NewPlanPrices] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('prices') }})
    r"""Prices for this plan. If the plan has phases, this includes prices across all phases of the plan."""
    default_invoice_memo: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('default_invoice_memo') }})
    r"""Free-form text which is available on the invoice PDF and the Orb invoice portal."""
    external_plan_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_plan_id') }})
    metadata: Optional[NewPlanMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    net_terms: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    r"""The net terms determines the difference between the invoice date and the issue date for the invoice. If you intend the invoice to be due on issue, set this to 0."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import requests as requests_http
from ..shared import subscriptions as shared_subscriptions
from typing import Optional



@dataclasses.dataclass
class ListCouponSubscriptionsRequest:
    coupon_id: str = dataclasses.field(metadata={'path_param': { 'field_name': 'coupon_id', 'style': 'simple', 'explode': False }})
    cursor: Optional[str] = dataclasses.field(default=None, metadata={'query_param': { 'field_name': 'cursor', 'style': 'form', 'explode': True }})
    limit: Optional[int] = dataclasses.field(default=20, metadata={'query_param': { 'field_name': 'limit', 'style': 'form', 'explode': True }})
    




@dataclasses.dataclass
class ListCouponSubscriptionsResponse:
    content_type: str = dataclasses.field()
    r"""HTTP response content type for this operation"""
    status_code: int = dataclasses.field()
    r"""HTTP response status code for this operation"""
    raw_response: Optional[requests_http.Response] = dataclasses.field(default=None)
    r"""Raw HTTP response; suitable for custom response parsing"""
    subscriptions: Optional[shared_subscriptions.Subscriptions] = dataclasses.field(default=None)
    r"""OK"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from orb import utils



@dataclasses.dataclass
class FourHundredAndNineError(Exception):
    

    def __str__(self) -> str:
        return utils.marshal_json(self)



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import sublineitemgrouping as shared_sublineitemgrouping
from ..shared import sublineitemmatrixconfig as shared_sublineitemmatrixconfig
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class MatrixSubLineItemQuantity:
    pass

class MatrixSubLineItemType(str, Enum):
    MATRIX = 'matrix'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class MatrixSubLineItem:
    amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('amount') }})
    r"""The total amount for this sub line item."""
    grouping: Optional[shared_sublineitemgrouping.SubLineItemGrouping] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('grouping') }})
    matrix_config: shared_sublineitemmatrixconfig.SubLineItemMatrixConfig = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('matrix_config') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    quantity: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('quantity') }})
    type: MatrixSubLineItemType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .accountingprovider import *
from .accountingproviderconfig import *
from .accountingsyncconfiguration import *
from .addamendmentcreditledgerentryrequestparams import *
from .adddecrementcreditledgerentryrequestparams import *
from .addeditpriceintervalparams import *
from .addexpirationchangecreditledgerentryrequestparams import *
from .addincrementcreditledgerentryrequestparams import *
from .addpriceintervalparams import *
from .address import *
from .addressinput import *
from .addvoidcreditledgerentryrequestparams import *
from .affectedblock import *
from .aggregatedcost import *
from .amendedevent import *
from .amendedusage import *
from .amendeventresult import *
from .amendmentledgerentry import *
from .amountdiscount import *
from .amountdiscountcreationparams import *
from .amountdiscountinterval import *
from .autocollection import *
from .backfill import *
from .backfills import *
from .backfillstatus import *
from .billablemetric import *
from .billablemetrics import *
from .billablemetricsimple import *
from .billablemetrictiny import *
from .billingcyclealignment import *
from .bpsprice import *
from .bulkbpsprice import *
from .bulkprice import *
from .cancelsubscriptionparams import *
from .changeoption import *
from .coupon import *
from .couponredemption import *
from .coupons import *
from .createinvoicelineitemparams import *
from .createoneoffinvoiceparams import *
from .creditblockexpiryledgerentry import *
from .creditledgerentries import *
from .creditnote import *
from .creditnotelineitem import *
from .creditnoteminified import *
from .creditnotes import *
from .creditnotesublineitem import *
from .creditnotesummary import *
from .customer import *
from .customerbalancetransaction import *
from .customerbalancetransactions import *
from .customercosts import *
from .customercreditbalance import *
from .customercreditbalances import *
from .customerminified import *
from .customers import *
from .customertaxid import *
from .debug import *
from .decrementledgerentry import *
from .deletecustomerresponse import *
from .deprecatedeventresult import *
from .discount import *
from .duplicateconstraintviolationerror import *
from .duplicateresourcecreationerror import *
from .editcustomer import *
from .editplan import *
from .editpriceintervalparams import *
from .event import *
from .events import *
from .eventsearchcriteria import *
from .expirationchangeledgerentry import *
from .featurenotavailableerror import *
from .fivetranexampleprice import *
from .fixedfeequantitychange import *
from .fixedfeequantityscheduleentry import *
from .fixedfeequantitytransition import *
from .groupedsubscriptionusage import *
from .groupedsubscriptionusagerecord import *
from .idempotencyconflict import *
from .idempotencykeyvalidationerror import *
from .idempotencyrequestmismatch import *
from .incrementcreditledgerentryinvoicesettings import *
from .incrementledgerentry import *
from .ingestevent import *
from .ingestionresponse import *
from .ingestrequestbody import *
from .invoice import *
from .invoicelineitem import *
from .invoicelineitemparams import *
from .invoiceminified import *
from .invoices import *
from .item import *
from .itemexternalconnection import *
from .items import *
from .itemslim import *
from .markaspaidrequestparams import *
from .matrixprice import *
from .matrixsublineitem import *
from .maximum import *
from .maximuminterval import *
from .metricgroup import *
from .minimum import *
from .minimuminterval import *
from .newaccountingsyncconfiguration import *
from .newbackfill import *
from .newbillablemetric import *
from .newbpsprice import *
from .newbulkbpsprice import *
from .newbulkprice import *
from .newcoupon import *
from .newcustomer import *
from .newcustomerbalancetransaction import *
from .newmatrixprice import *
from .newpackageprice import *
from .newpackagewithallocationprice import *
from .newplan import *
from .newreportingconfiguration import *
from .newsubscription import *
from .newthresholdtotalamountprice import *
from .newtieredbpsprice import *
from .newtieredpackageprice import *
from .newtieredprice import *
from .newtieredwithminimumprice import *
from .newunitprice import *
from .othersublineitem import *
from .overridebpsprice import *
from .overridebulkbpsprice import *
from .overridebulkprice import *
from .overridefivetranexampleprice import *
from .overridematrixprice import *
from .overridepackageprice import *
from .overridepackagewithallocationprice import *
from .overridetestratingfunctionprice import *
from .overridethresholdtotalamountprice import *
from .overridetieredbpsprice import *
from .overridetieredpackageprice import *
from .overridetieredprice import *
from .overridetieredwithminimumprice import *
from .overrideunitprice import *
from .packageprice import *
from .packagewithallocationprice import *
from .paginationmetadata import *
from .percentagediscount import *
from .percentagediscountcreationparams import *
from .percentagediscountinterval import *
from .perpricecost import *
from .phaseoverride import *
from .pingresponse import *
from .plan import *
from .planminified import *
from .planphase import *
from .plans import *
from .pricegroup import *
from .priceinterval import *
from .priceintervalfixedfeequantitytransitionparams import *
from .prices import *
from .product import *
from .reportingconfiguration import *
from .requesttoolargeerror import *
from .requestvalidationerror import *
from .resourcenotfounderror import *
from .resourcetoolargeerror import *
from .security import *
from .sublineitemgrouping import *
from .sublineitemmatrixconfig import *
from .subscription import *
from .subscriptioncosts import *
from .subscriptionminified import *
from .subscriptionplanchange import *
from .subscriptions import *
from .subscriptionscheduleitem import *
from .subscriptionscheduleitems import *
from .subscriptiontrialinfo import *
from .taxamount import *
from .testratingfunctionprice import *
from .thresholdtotalamountprice import *
from .tierconfig import *
from .tieredbpsprice import *
from .tieredpackageprice import *
from .tieredprice import *
from .tieredwithminimumprice import *
from .tiersublineitem import *
from .trialconfig import *
from .trialdiscount import *
from .triggersubscriptionphaseparams import *
from .ungroupedsubscriptionusage import *
from .ungroupedsubscriptionusagerecord import *
from .unitprice import *
from .unschedulefixedfeequantitychangeparams import *
from .upcominginvoice import *
from .updateevent import *
from .urlnotfound import *
from .usage import *
from .usagediscount import *
from .usagediscountcreationparams import *
from .usagediscountinterval import *
from .validationerror import *
from .voidinitiatedledgerentry import *
from .voidledgerentry import *

__all__ = ["AccountingProvider","AccountingProviderConfig","AccountingProviderProviderType","AccountingSyncConfiguration","AddAmendmentCreditLedgerEntryRequestParams","AddAmendmentCreditLedgerEntryRequestParamsAmount","AddAmendmentCreditLedgerEntryRequestParamsEntryType","AddAmendmentCreditLedgerEntryRequestParamsMetadata","AddDecrementCreditLedgerEntryRequestParams","AddDecrementCreditLedgerEntryRequestParamsAmount","AddDecrementCreditLedgerEntryRequestParamsEntryType","AddDecrementCreditLedgerEntryRequestParamsMetadata","AddEditPriceIntervalParams","AddExpirationChangeCreditLedgerEntryRequestParams","AddExpirationChangeCreditLedgerEntryRequestParamsAmount","AddExpirationChangeCreditLedgerEntryRequestParamsEntryType","AddExpirationChangeCreditLedgerEntryRequestParamsMetadata","AddIncrementCreditLedgerEntryRequestParams","AddIncrementCreditLedgerEntryRequestParamsAmount","AddIncrementCreditLedgerEntryRequestParamsEntryType","AddIncrementCreditLedgerEntryRequestParamsMetadata","AddPriceIntervalParams","AddPriceIntervalParamsDiscounts","AddPriceIntervalParamsEndDate","AddPriceIntervalParamsEndDateEndDate","AddPriceIntervalParamsStartDate","AddPriceIntervalParamsStartDateStartDate","AddVoidCreditLedgerEntryRequestParams","AddVoidCreditLedgerEntryRequestParamsAmount","AddVoidCreditLedgerEntryRequestParamsEntryType","AddVoidCreditLedgerEntryRequestParamsMetadata","AddVoidCreditLedgerEntryRequestParamsVoidReason","Address","AddressInput","AffectedBlock","AggregatedCost","AmendEventResult","AmendedEvent","AmendedEventProperties","AmendedUsage","AmendmentLedgerEntry","AmendmentLedgerEntryAmount","AmendmentLedgerEntryEndingBalance","AmendmentLedgerEntryEntryStatus","AmendmentLedgerEntryEntryType","AmendmentLedgerEntryStartingBalance","AmountDiscount","AmountDiscountCreationParams","AmountDiscountCreationParamsAmountDiscount","AmountDiscountCreationParamsDiscountType","AmountDiscountDiscountType","AmountDiscountInterval","AmountDiscountIntervalDiscountType","AutoCollection","Backfill","BackfillStatus","Backfills","BillableMetric","BillableMetricSimple","BillableMetricStatus","BillableMetricTiny","BillableMetrics","BillingCycleAlignment","BpsPrice","BpsPriceCadence","BpsPriceDiscount","BpsPriceModelType","BpsPricePriceType","BulkBpsPrice","BulkBpsPriceCadence","BulkBpsPriceDiscount","BulkBpsPriceModelType","BulkBpsPricePriceType","BulkPrice","BulkPriceCadence","BulkPriceDiscount","BulkPriceModelType","BulkPricePriceType","CancelSubscriptionParams","CancelSubscriptionParamsCancelOption","ChangeOption","Coupon","CouponDiscount","CouponRedemption","Coupons","CreateInvoiceLineItemParams","CreateInvoiceLineItemParamsQuantity","CreateOneOffInvoiceParams","CreditBlockExpiryLedgerEntry","CreditBlockExpiryLedgerEntryAmount","CreditBlockExpiryLedgerEntryEndingBalance","CreditBlockExpiryLedgerEntryEntryStatus","CreditBlockExpiryLedgerEntryEntryType","CreditBlockExpiryLedgerEntryStartingBalance","CreditLedgerEntries","CreditNote","CreditNoteDiscounts","CreditNoteLineItem","CreditNoteLineItemDiscounts","CreditNoteLineItemTaxAmounts","CreditNoteMaximumAmountAdjustment","CreditNoteMinified","CreditNoteReason","CreditNoteSubLineItem","CreditNoteSummary","CreditNoteType","CreditNotes","Customer","CustomerBalanceTransaction","CustomerBalanceTransactionAction","CustomerBalanceTransactionType","CustomerBalanceTransactions","CustomerCosts","CustomerCreditBalance","CustomerCreditBalanceBalance","CustomerCreditBalances","CustomerMinified","CustomerPaymentProvider","CustomerTaxID","Customers","Debug","DecrementLedgerEntry","DecrementLedgerEntryAmount","DecrementLedgerEntryEndingBalance","DecrementLedgerEntryEntryStatus","DecrementLedgerEntryEntryType","DecrementLedgerEntryStartingBalance","DeleteCustomerResponse","DeprecatedEventResult","Discount","DiscountDiscountType","DiscountPercentageDiscount","DiscountUsageDiscount","DuplicateConstraintViolationError","DuplicateConstraintViolationErrorHeaders","DuplicateConstraintViolationErrorStatus","DuplicateConstraintViolationErrorType","DuplicateResourceCreationError","DuplicateResourceCreationErrorHeaders","DuplicateResourceCreationErrorStatus","DuplicateResourceCreationErrorType","EditCustomer","EditCustomerMetadata","EditCustomerPaymentProvider","EditPlan","EditPlanMetadata","EditPriceIntervalParams","EditPriceIntervalParamsEndDate","EditPriceIntervalParamsEndDateEndDate","EditPriceIntervalParamsStartDate","EditPriceIntervalParamsStartDateStartDate","Event","EventProperties","EventSearchCriteria","Events","ExpirationChangeLedgerEntry","ExpirationChangeLedgerEntryAmount","ExpirationChangeLedgerEntryEndingBalance","ExpirationChangeLedgerEntryEntryStatus","ExpirationChangeLedgerEntryEntryType","ExpirationChangeLedgerEntryStartingBalance","FeatureNotAvailableError","FeatureNotAvailableErrorHeaders","FeatureNotAvailableErrorStatus","FeatureNotAvailableErrorType","FivetranExamplePrice","FivetranExamplePriceCadence","FivetranExamplePriceDiscount","FivetranExamplePriceModelType","FivetranExamplePricePriceType","FixedFeeQuantityChange","FixedFeeQuantityChangeChangeOption","FixedFeeQuantityChangeQuantity","FixedFeeQuantityScheduleEntry","FixedFeeQuantityTransition","GroupedSubscriptionUsage","GroupedSubscriptionUsageRecord","GroupedSubscriptionUsageRecordViewMode","IdempotencyConflict","IdempotencyConflictHeaders","IdempotencyConflictStatus","IdempotencyConflictType","IdempotencyKeyValidationError","IdempotencyKeyValidationErrorHeaders","IdempotencyKeyValidationErrorStatus","IdempotencyKeyValidationErrorType","IdempotencyRequestMismatch","IdempotencyRequestMismatchHeaders","IdempotencyRequestMismatchStatus","IdempotencyRequestMismatchType","IncrementCreditLedgerEntryInvoiceSettings","IncrementLedgerEntry","IncrementLedgerEntryAmount","IncrementLedgerEntryEndingBalance","IncrementLedgerEntryEntryStatus","IncrementLedgerEntryEntryType","IncrementLedgerEntryStartingBalance","IngestEvent","IngestEventProperties","IngestRequestBody","IngestionResponse","Invoice","InvoiceDiscount","InvoiceDiscounts","InvoiceLineItem","InvoiceLineItemDiscount","InvoiceLineItemParams","InvoiceLineItemParamsModelType","InvoiceLineItemParamsQuantity","InvoiceLineItemQuantity","InvoiceMetadata","InvoiceMinified","InvoiceStatus","Invoices","Item","ItemExternalConnection","ItemExternalConnectionExternalConnectionName","ItemExternalConnectionExternalConnectionNameExternalConnectionName","ItemExternalConnectionExternalConnectionNameExternalConnectionName1","ItemSlim","Items","MarkAsPaidRequestParams","MatrixPrice","MatrixPriceCadence","MatrixPriceDiscount","MatrixPriceModelType","MatrixPricePriceType","MatrixSubLineItem","MatrixSubLineItemQuantity","MatrixSubLineItemType","Maximum","MaximumInterval","MetricGroup","Minimum","MinimumInterval","NewAccountingSyncConfiguration","NewBackfill","NewBillableMetric","NewBillableMetricMetadata","NewBpsPrice","NewBpsPriceCadence","NewBpsPriceFixedPriceQuantity","NewBpsPriceModelType","NewBulkBpsPrice","NewBulkBpsPriceCadence","NewBulkBpsPriceFixedPriceQuantity","NewBulkBpsPriceModelType","NewBulkPrice","NewBulkPriceCadence","NewBulkPriceFixedPriceQuantity","NewBulkPriceModelType","NewCoupon","NewCouponDiscount","NewCustomer","NewCustomerBalanceTransaction","NewCustomerBalanceTransactionType","NewCustomerMetadata","NewCustomerPaymentProvider","NewMatrixPrice","NewMatrixPriceCadence","NewMatrixPriceFixedPriceQuantity","NewMatrixPriceModelType","NewPackagePrice","NewPackagePriceCadence","NewPackagePriceFixedPriceQuantity","NewPackagePriceModelType","NewPackageWithAllocationPrice","NewPackageWithAllocationPriceCadence","NewPackageWithAllocationPriceFixedPriceQuantity","NewPackageWithAllocationPriceModelType","NewPlan","NewPlanMetadata","NewPlanPrices","NewReportingConfiguration","NewSubscription","NewSubscriptionCreditsOverageRate","NewSubscriptionExternalMarketplace","NewSubscriptionMetadata","NewSubscriptionPriceOverrides","NewThresholdTotalAmountPrice","NewThresholdTotalAmountPriceCadence","NewThresholdTotalAmountPriceFixedPriceQuantity","NewThresholdTotalAmountPriceModelType","NewTieredBpsPrice","NewTieredBpsPriceCadence","NewTieredBpsPriceFixedPriceQuantity","NewTieredBpsPriceModelType","NewTieredPackagePrice","NewTieredPackagePriceCadence","NewTieredPackagePriceFixedPriceQuantity","NewTieredPackagePriceModelType","NewTieredPrice","NewTieredPriceCadence","NewTieredPriceFixedPriceQuantity","NewTieredPriceModelType","NewTieredWithMinimumPrice","NewTieredWithMinimumPriceCadence","NewTieredWithMinimumPriceFixedPriceQuantity","NewTieredWithMinimumPriceModelType","NewUnitPrice","NewUnitPriceCadence","NewUnitPriceFixedPriceQuantity","NewUnitPriceModelType","OtherSubLineItem","OtherSubLineItemQuantity","OtherSubLineItemType","OverrideBpsPrice","OverrideBpsPriceFixedPriceQuantity","OverrideBpsPriceModelType","OverrideBulkBpsPrice","OverrideBulkBpsPriceFixedPriceQuantity","OverrideBulkBpsPriceModelType","OverrideBulkPrice","OverrideBulkPriceFixedPriceQuantity","OverrideBulkPriceModelType","OverrideFivetranExamplePrice","OverrideFivetranExamplePriceFixedPriceQuantity","OverrideFivetranExamplePriceModelType","OverrideMatrixPrice","OverrideMatrixPriceFixedPriceQuantity","OverrideMatrixPriceModelType","OverridePackagePrice","OverridePackagePriceFixedPriceQuantity","OverridePackagePriceModelType","OverridePackageWithAllocationPrice","OverridePackageWithAllocationPriceFixedPriceQuantity","OverridePackageWithAllocationPriceModelType","OverrideTestRatingFunctionPrice","OverrideTestRatingFunctionPriceFixedPriceQuantity","OverrideTestRatingFunctionPriceModelType","OverrideThresholdTotalAmountPrice","OverrideThresholdTotalAmountPriceFixedPriceQuantity","OverrideThresholdTotalAmountPriceModelType","OverrideTieredBpsPrice","OverrideTieredBpsPriceFixedPriceQuantity","OverrideTieredBpsPriceModelType","OverrideTieredPackagePrice","OverrideTieredPackagePriceFixedPriceQuantity","OverrideTieredPackagePriceModelType","OverrideTieredPrice","OverrideTieredPriceFixedPriceQuantity","OverrideTieredPriceModelType","OverrideTieredWithMinimumPrice","OverrideTieredWithMinimumPriceFixedPriceQuantity","OverrideTieredWithMinimumPriceModelType","OverrideUnitPrice","OverrideUnitPriceFixedPriceQuantity","OverrideUnitPriceModelType","PackagePrice","PackagePriceCadence","PackagePriceDiscount","PackagePriceModelType","PackagePricePriceType","PackageWithAllocationPrice","PackageWithAllocationPriceCadence","PackageWithAllocationPriceDiscount","PackageWithAllocationPriceModelType","PackageWithAllocationPricePriceType","PaginationMetadata","PerPriceCost","PerPriceCostQuantity","PercentageDiscount","PercentageDiscountCreationParams","PercentageDiscountCreationParamsDiscountType","PercentageDiscountCreationParamsPercentageDiscount","PercentageDiscountDiscountType","PercentageDiscountInterval","PercentageDiscountIntervalDiscountType","PercentageDiscountIntervalPercentageDiscount","PercentageDiscountPercentageDiscount","PhaseOverride","PingResponse","Plan","PlanDiscount","PlanMetadata","PlanMinified","PlanPhase","PlanPhaseDiscount","PlanPhaseDurationUnit","PlanStatus","Plans","PriceGroup","PriceInterval","PriceIntervalFixedFeeQuantityTransitionParams","Prices","Product","ReportingConfiguration","RequestTooLargeError","RequestTooLargeErrorHeaders","RequestTooLargeErrorStatus","RequestTooLargeErrorType","RequestValidationError","RequestValidationErrorHeaders","RequestValidationErrorStatus","RequestValidationErrorType","ResourceNotFoundError","ResourceNotFoundErrorHeaders","ResourceNotFoundErrorStatus","ResourceNotFoundErrorType","ResourceTooLargeError","ResourceTooLargeErrorHeaders","ResourceTooLargeErrorStatus","ResourceTooLargeErrorType","Security","SubLineItemGrouping","SubLineItemMatrixConfig","Subscription","SubscriptionCosts","SubscriptionDiscountIntervals","SubscriptionMetadata","SubscriptionMinified","SubscriptionPlanChange","SubscriptionPlanChangeCreditsOverageRate","SubscriptionPlanChangePriceOverrides","SubscriptionScheduleItem","SubscriptionScheduleItems","SubscriptionStatus","SubscriptionTrialInfo","Subscriptions","TaxAmount","TestRatingFunctionPrice","TestRatingFunctionPriceCadence","TestRatingFunctionPriceDiscount","TestRatingFunctionPriceModelType","TestRatingFunctionPricePriceType","ThresholdTotalAmountPrice","ThresholdTotalAmountPriceCadence","ThresholdTotalAmountPriceDiscount","ThresholdTotalAmountPriceModelType","ThresholdTotalAmountPricePriceType","TierConfig","TierConfigFirstUnit","TierConfigLastUnit","TierSubLineItem","TierSubLineItemQuantity","TierSubLineItemType","TieredBpsPrice","TieredBpsPriceCadence","TieredBpsPriceDiscount","TieredBpsPriceModelType","TieredBpsPricePriceType","TieredPackagePrice","TieredPackagePriceCadence","TieredPackagePriceDiscount","TieredPackagePriceModelType","TieredPackagePricePriceType","TieredPrice","TieredPriceCadence","TieredPriceDiscount","TieredPriceModelType","TieredPricePriceType","TieredWithMinimumPrice","TieredWithMinimumPriceCadence","TieredWithMinimumPriceDiscount","TieredWithMinimumPriceModelType","TieredWithMinimumPricePriceType","TrialConfig","TrialConfigTrialPeriodUnit","TrialDiscount","TrialDiscountDiscountType","TrialDiscountTrialPercentageDiscount","TriggerSubscriptionPhaseParams","URLNotFound","URLNotFoundHeaders","URLNotFoundStatus","URLNotFoundType","UngroupedSubscriptionUsage","UngroupedSubscriptionUsageRecord","UngroupedSubscriptionUsageRecordViewMode","UnitPrice","UnitPriceCadence","UnitPriceDiscount","UnitPriceModelType","UnitPricePriceType","UnscheduleFixedFeeQuantityChangeParams","UpcomingInvoice","UpcomingInvoiceDiscount","UpcomingInvoiceDiscounts","UpcomingInvoiceMetadata","UpcomingInvoiceStatus","UpdateEvent","UpdateEventProperties","Usage","UsageDiscount","UsageDiscountCreationParams","UsageDiscountCreationParamsDiscountType","UsageDiscountCreationParamsUsageDiscount","UsageDiscountDiscountType","UsageDiscountInterval","UsageDiscountIntervalDiscountType","UsageDiscountIntervalUsageDiscount","UsageDiscountUsageDiscount","ValidationError","VoidInitiatedLedgerEntry","VoidInitiatedLedgerEntryAmount","VoidInitiatedLedgerEntryEndingBalance","VoidInitiatedLedgerEntryEntryStatus","VoidInitiatedLedgerEntryEntryType","VoidInitiatedLedgerEntryStartingBalance","VoidInitiatedLedgerEntryVoidAmount","VoidLedgerEntry","VoidLedgerEntryAmount","VoidLedgerEntryEndingBalance","VoidLedgerEntryEntryStatus","VoidLedgerEntryEntryType","VoidLedgerEntryStartingBalance","VoidLedgerEntryVoidAmount"]



"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CreditNoteSummary:
    credit_note_number: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('credit_note_number') }})
    id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('id') }})
    reason: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('reason') }})
    total: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('total') }})
    type: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('type') }})
    voided_at: Optional[datetime] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('voided_at'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""If the credit note has a status of `void`, this gives a timestamp when the credit note was voided."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from ..shared import addpriceintervalparams as shared_addpriceintervalparams
from ..shared import editpriceintervalparams as shared_editpriceintervalparams
from dataclasses_json import Undefined, dataclass_json
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class AddEditPriceIntervalParams:
    add: Optional[list[shared_addpriceintervalparams.AddPriceIntervalParams]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('add'), 'exclude': lambda f: f is None }})
    r"""A list of price intervals to add to the subscription."""
    edit: Optional[list[shared_editpriceintervalparams.EditPriceIntervalParams]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('edit'), 'exclude': lambda f: f is None }})
    r"""A list of price intervals to edit on the subscription."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import discount as shared_discount
from ..shared import overridebpsprice as shared_overridebpsprice
from ..shared import overridebulkbpsprice as shared_overridebulkbpsprice
from ..shared import overridebulkprice as shared_overridebulkprice
from ..shared import overridefivetranexampleprice as shared_overridefivetranexampleprice
from ..shared import overridematrixprice as shared_overridematrixprice
from ..shared import overridepackageprice as shared_overridepackageprice
from ..shared import overridepackagewithallocationprice as shared_overridepackagewithallocationprice
from ..shared import overridetestratingfunctionprice as shared_overridetestratingfunctionprice
from ..shared import overridethresholdtotalamountprice as shared_overridethresholdtotalamountprice
from ..shared import overridetieredbpsprice as shared_overridetieredbpsprice
from ..shared import overridetieredpackageprice as shared_overridetieredpackageprice
from ..shared import overridetieredprice as shared_overridetieredprice
from ..shared import overridetieredwithminimumprice as shared_overridetieredwithminimumprice
from ..shared import overrideunitprice as shared_overrideunitprice
from ..shared import phaseoverride as shared_phaseoverride
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from enum import Enum
from orb import utils
from typing import Optional, Union



@dataclasses.dataclass
class NewSubscriptionCreditsOverageRate:
    pass

class NewSubscriptionExternalMarketplace(str, Enum):
    GOOGLE = 'google'
    AWS = 'aws'
    AZURE = 'azure'



@dataclasses.dataclass
class NewSubscriptionMetadata:
    pass



@dataclasses.dataclass
class NewSubscriptionPriceOverrides:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class NewSubscription:
    align_billing_with_subscription_start_date: Optional[bool] = dataclasses.field(default=False, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('align_billing_with_subscription_start_date'), 'exclude': lambda f: f is None }})
    auto_collection: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('auto_collection') }})
    aws_region: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('aws_region') }})
    coupon_redemption_code: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('coupon_redemption_code') }})
    credits_overage_rate: Optional[Union[float, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('credits_overage_rate') }})
    customer_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customer_id') }})
    default_invoice_memo: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('default_invoice_memo') }})
    discount: Optional[shared_discount.Discount] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount') }})
    r"""The subscription's override discount for the plan."""
    end_date: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('end_date'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse }})
    external_customer_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_customer_id') }})
    external_marketplace: Optional[NewSubscriptionExternalMarketplace] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_marketplace') }})
    external_marketplace_reporting_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_marketplace_reporting_id') }})
    external_plan_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_plan_id') }})
    r"""The external_plan_id of the plan that the given subscription should be switched to. Note that either this property or `plan_id` must be specified."""
    initial_phase_order: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('initial_phase_order') }})
    invoicing_threshold: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoicing_threshold') }})
    maximum_amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('maximum_amount') }})
    r"""The subscription's override maximum amount for the plan."""
    metadata: Optional[NewSubscriptionMetadata] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('metadata') }})
    minimum_amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    r"""The subscription's override minimum amount for the plan."""
    net_terms: Optional[int] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    per_credit_overage_amount: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('per_credit_overage_amount') }})
    phase_overrides: Optional[list[shared_phaseoverride.PhaseOverride]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('phase_overrides') }})
    plan_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('plan_id') }})
    r"""The plan that the given subscription should be switched to. Note that either this property or `external_plan_id` must be specified."""
    price_overrides: Optional[list[Union[shared_overrideunitprice.OverrideUnitPrice, shared_overridepackageprice.OverridePackagePrice, shared_overridematrixprice.OverrideMatrixPrice, shared_overridetieredprice.OverrideTieredPrice, shared_overridetieredbpsprice.OverrideTieredBpsPrice, shared_overridebpsprice.OverrideBpsPrice, shared_overridebulkbpsprice.OverrideBulkBpsPrice, shared_overridebulkprice.OverrideBulkPrice, shared_overridetestratingfunctionprice.OverrideTestRatingFunctionPrice, shared_overridefivetranexampleprice.OverrideFivetranExamplePrice, shared_overridethresholdtotalamountprice.OverrideThresholdTotalAmountPrice, shared_overridetieredpackageprice.OverrideTieredPackagePrice, shared_overridetieredwithminimumprice.OverrideTieredWithMinimumPrice, shared_overridepackagewithallocationprice.OverridePackageWithAllocationPrice]]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('price_overrides') }})
    r"""Optionally provide a list of overrides for prices on the plan"""
    start_date: Optional[datetime] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('start_date'), 'encoder': utils.datetimeisoformat(True), 'decoder': dateutil.parser.isoparse }})
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Union

class PercentageDiscountCreationParamsDiscountType(str, Enum):
    PERCENTAGE = 'percentage'



@dataclasses.dataclass
class PercentageDiscountCreationParamsPercentageDiscount:
    pass


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class PercentageDiscountCreationParams:
    discount_type: PercentageDiscountCreationParamsDiscountType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('discount_type') }})
    percentage_discount: Union[float, str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('percentage_discount') }})
    r"""Only available if discount_type is `percentage`. This is a number between 0 and 1."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
import dateutil.parser
from ..shared import invoicelineitemparams as shared_invoicelineitemparams
from dataclasses_json import Undefined, dataclass_json
from datetime import datetime
from orb import utils
from typing import Optional


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class CreateOneOffInvoiceParams:
    currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency') }})
    r"""An ISO 4217 currency string. Must be the same as the customer's currency if it is set."""
    invoice_date: datetime = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoice_date'), 'encoder': utils.datetimeisoformat(False), 'decoder': dateutil.parser.isoparse }})
    r"""Optional invoice date to set. Must be in the past, if not set, `invoice_date` is set to the current time in the customer's timezone."""
    line_items: list[shared_invoicelineitemparams.InvoiceLineItemParams] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('line_items') }})
    net_terms: int = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('net_terms') }})
    r"""Determines the difference between the invoice issue date for subscription invoices as the date that they are due. A value of '0' here represents that the invoice is due on issue, whereas a value of 30 represents that the customer has 30 days to pay the invoice."""
    customer_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('customer_id') }})
    r"""The id of the `Customer` to create this invoice for. One of `customer_id` and `external_customer_id` are required."""
    external_customer_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_customer_id') }})
    r"""The `external_customer_id` of the `Customer` to create this invoice for. One of `customer_id` and `external_customer_id` are required."""
    memo: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('memo') }})
    r"""An optional memo to attach to the invoice."""
    will_auto_issue: Optional[bool] = dataclasses.field(default=False, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('will_auto_issue'), 'exclude': lambda f: f is None }})
    r"""When true, this invoice will automatically be issued upon creation. When false, the resulting invoice will require manual review to issue. Defaulted to false."""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from .sdkconfiguration import SDKConfiguration
from orb import utils
from orb.models import errors, operations, shared
from typing import Optional

class Invoice:
    r"""An [`Invoice`](../guides/concepts#invoice) is a fundamental billing entity, representing the request for payment for
    a single subscription. This includes a set of line items, which correspond to prices in the subscription's plan and
    can represent fixed recurring fees or usage-based fees. They are generated at the end of a billing period, or as
    the result of an action, such as a cancellation.
    """
    sdk_configuration: SDKConfiguration

    def __init__(self, sdk_config: SDKConfiguration) -> None:
        self.sdk_configuration = sdk_config
        
    
    def create(self, request: shared.CreateOneOffInvoiceParams) -> operations.CreateInvoiceResponse:
        r"""Create a one-off invoice
        This endpoint is used to create a one-off invoice for a customer.
        """
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/invoices'
        headers = {}
        req_content_type, data, form = utils.serialize_request_body(request, "request", False, True, 'json')
        if req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('POST', url, data=data, files=form, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.CreateInvoiceResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 201:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoice])
                res.invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def create_line_item(self, request: shared.CreateInvoiceLineItemParams) -> operations.CreateInvoiceLineItemResponse:
        r"""Create invoice line item
        This creates a one-off fixed fee invoice line item on an Invoice.
        This can only be done for invoices that are in a `draft` status.
        """
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/invoice_line_items'
        headers = {}
        req_content_type, data, form = utils.serialize_request_body(request, "request", False, True, 'json')
        if req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('POST', url, data=data, files=form, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.CreateInvoiceLineItemResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 201:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.InvoiceLineItem])
                res.invoice_line_item = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def fetch(self, invoice_id: str) -> operations.FetchInvoiceResponse:
        r"""Fetch invoice
        This endpoint is used to fetch an [`Invoice`](../guides/concepts#invoice) given an identifier.
        """
        request = operations.FetchInvoiceRequest(
            invoice_id=invoice_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.FetchInvoiceRequest, base_url, '/invoices/{invoice_id}', request)
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.FetchInvoiceResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoice])
                res.invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def fetch_upcoming(self, subscription_id: Optional[str] = None) -> operations.FetchUpcomingInvoiceResponse:
        r"""Fetch upcoming invoice
        This endpoint can be used to fetch the upcoming [invoice](../guides/concepts#invoice) for the current billing
        period given a subscription.
        """
        request = operations.FetchUpcomingInvoiceRequest(
            subscription_id=subscription_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/invoices/upcoming'
        headers = {}
        query_params = utils.get_query_params(operations.FetchUpcomingInvoiceRequest, request)
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, params=query_params, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.FetchUpcomingInvoiceResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.UpcomingInvoice])
                res.upcoming_invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def issue(self, invoice_id: str) -> operations.IssueInvoiceResponse:
        r"""Issue invoice
        This endpoint allows an eligible invoice to be issued manually. This is only possible with invoices where status is `draft`,
        `will_auto_issue` is true, and an `eligible_to_issue_at` is a time in the past. Issuing an invoice could possibly trigger
        side effects, some of which could be customer-visible (e.g. sending emails, auto-collecting payment,
        syncing the invoice to external providers, etc).
        """
        request = operations.IssueInvoiceRequest(
            invoice_id=invoice_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.IssueInvoiceRequest, base_url, '/invoices/{invoice_id}/issue', request)
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('POST', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.IssueInvoiceResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoice])
                res.invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def list(self, request: operations.ListInvoicesRequest) -> operations.ListInvoicesResponse:
        r"""List invoices
        This endpoint returns a list of all [`Invoice`](../guides/concepts#invoice)s for an account in a list format.

        The list of invoices is ordered starting from the most recently issued invoice date. The response also includes
        [`pagination_metadata`](../reference/pagination), which lets the caller retrieve the next page of results if they
        exist.

        By default, this only returns invoices that are `issued`, `paid`, or `synced`.
        """
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = base_url + '/invoices'
        headers = {}
        query_params = utils.get_query_params(operations.ListInvoicesRequest, request)
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('GET', url, params=query_params, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.ListInvoicesResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoices])
                res.invoices = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def mark_invoice_as_paid(self, invoice_id: str, mark_as_paid_request_params: Optional[shared.MarkAsPaidRequestParams] = None) -> operations.MarkInvoiceAsPaidResponse:
        r"""Mark invoice as paid
        This endpoint allows an invoice's status to be set the `paid` status. This can only be done to invoices that are in
        the `issued` status.
        """
        request = operations.MarkInvoiceAsPaidRequest(
            invoice_id=invoice_id,
            mark_as_paid_request_params=mark_as_paid_request_params,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.MarkInvoiceAsPaidRequest, base_url, '/invoices/{invoice_id}/mark_paid', request)
        headers = {}
        req_content_type, data, form = utils.serialize_request_body(request, "mark_as_paid_request_params", False, True, 'json')
        if req_content_type not in ('multipart/form-data', 'multipart/mixed'):
            headers['content-type'] = req_content_type
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('POST', url, data=data, files=form, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.MarkInvoiceAsPaidResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoice])
                res.invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    
    def void(self, invoice_id: str) -> operations.VoidInvoiceResponse:
        r"""Void invoice
        This endpoint allows an invoice's status to be set the `void` status. This can only be done to invoices that are in
        the `issued` status.

        If the associated invoice has used the customer balance to change the amount due, the customer balance operation
        will be reverted. For example, if the invoice used $10 of customer balance, that amount will be added back to the
        customer balance upon voiding.
        """
        request = operations.VoidInvoiceRequest(
            invoice_id=invoice_id,
        )
        
        base_url = utils.template_url(*self.sdk_configuration.get_server_details())
        
        url = utils.generate_url(operations.VoidInvoiceRequest, base_url, '/invoices/{invoice_id}/void', request)
        headers = {}
        headers['Accept'] = 'application/json'
        headers['user-agent'] = self.sdk_configuration.user_agent
        
        client = self.sdk_configuration.security_client
        
        http_res = client.request('POST', url, headers=headers)
        content_type = http_res.headers.get('Content-Type')

        res = operations.VoidInvoiceResponse(status_code=http_res.status_code, content_type=content_type, raw_response=http_res)
        
        if http_res.status_code == 200:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, Optional[shared.Invoice])
                res.invoice = out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 400:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 401:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndOneError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 404:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndFourError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 409:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 413:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndThirteenError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 429:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FourHundredAndTwentyNineError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code >= 400 and http_res.status_code < 500 or http_res.status_code >= 500 and http_res.status_code < 600:
            raise errors.SDKError('API error occurred', http_res.status_code, http_res.text, http_res)
        elif http_res.status_code == 500:
            if utils.match_content_type(content_type, 'application/json'):
                out = utils.unmarshal_json(http_res.text, errors.FiveHundredError)
                out.raw_response = http_res
                raise out
            else:
                raise errors.SDKError(f'unknown content-type received: {content_type}', http_res.status_code, http_res.text, http_res)

        return res

    


"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from enum import Enum
from orb import utils
from typing import Any, Optional, Union

class NewThresholdTotalAmountPriceCadence(str, Enum):
    r"""The cadence to bill for this price on."""
    ANNUAL = 'annual'
    MONTHLY = 'monthly'
    QUARTERLY = 'quarterly'



@dataclasses.dataclass
class NewThresholdTotalAmountPriceFixedPriceQuantity:
    pass

class NewThresholdTotalAmountPriceModelType(str, Enum):
    THRESHOLD_TOTAL_AMOUNT = 'threshold_total_amount'


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class NewThresholdTotalAmountPrice:
    cadence: NewThresholdTotalAmountPriceCadence = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('cadence') }})
    r"""The cadence to bill for this price on."""
    currency: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('currency') }})
    r"""An ISO 4217 currency string for which this price is billed in."""
    item_id: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('item_id') }})
    r"""The id of the item the plan will be associated with."""
    model_type: NewThresholdTotalAmountPriceModelType = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('model_type') }})
    name: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('name') }})
    r"""The name of the price."""
    threshold_total_amount_config: dict[str, Any] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('threshold_total_amount_config') }})
    billable_metric_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billable_metric_id') }})
    r"""The id of the billable metric for the price. Only needed if the price is usage-based."""
    billed_in_advance: Optional[bool] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('billed_in_advance') }})
    r"""If the Price represents a fixed cost, the price will be billed in-advance if this is true, and in-arrears if this is false."""
    external_price_id: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('external_price_id') }})
    r"""An alias for the price."""
    fixed_price_quantity: Optional[Union[float, str]] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('fixed_price_quantity') }})
    r"""If the Price represents a fixed cost, this represents the quantity of units applied."""
    invoice_grouping_key: Optional[str] = dataclasses.field(default=None, metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('invoice_grouping_key') }})
    r"""The property used to group this price on an invoice"""
    




"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
import dataclasses
from dataclasses_json import Undefined, dataclass_json
from orb import utils


@dataclass_json(undefined=Undefined.EXCLUDE)

@dataclasses.dataclass
class Minimum:
    applies_to_price_ids: list[str] = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('applies_to_price_ids') }})
    r"""List of price_ids that this minimum amount applies to. For plan/plan phase minimums, this can be a subset of prices."""
    minimum_amount: str = dataclasses.field(metadata={'dataclasses_json': { 'letter_case': utils.get_field_name('minimum_amount') }})
    r"""Minimum amount applied"""
    

